package dev.fyloz.trial.colorrecipesexplorer.core.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.*;
import org.hibernate.validator.constraints.Length;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.List;

@Entity
@Table(name = "recipes")
@Data
@EqualsAndHashCode(callSuper = false)
@RequiredArgsConstructor
@NoArgsConstructor
public class Recipe extends BeanModel implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE)
    private Integer recipeID = 0;

    @NonNull
    @NotNull
    @Length(min = 2)
    private String recipeCode;

    @NonNull
    @NotNull
    @ManyToOne
    private Company company;

    @NonNull
    @NotNull
    private String recipeDescription;

    @NonNull
    @NotNull
    private int sample;

    private String approbationDate;

    private String remark;

    private String note;

    @JsonIgnore
    @OneToMany(mappedBy = "recipe", cascade = CascadeType.ALL)
    private List<Mix> recipeMixes;

    @OneToMany(mappedBy = "recipe", cascade = CascadeType.ALL, fetch = FetchType.LAZY)
    private List<RecipeStep> recipeSteps;

    @Override
    public Integer getID() {
        return recipeID;
    }

    public Material getBase() {
        if (recipeMixes.isEmpty() || recipeMixes.stream().allMatch(m -> m.getMixQuantities().isEmpty())) return null;

        Material base = recipeMixes
                .stream()
                .map(mix -> mix
                        .getMixQuantities()
                        .stream()
                        .filter(mq -> mq.getMaterial().getMaterialType().getMaterialTypeName().equals(MaterialType.BASE_MATERIAL_TYPE_NAME))
                        .findFirst().get()
                )
                .findFirst().orElse(recipeMixes
                        .stream()
                        .filter(m -> !m.getMixQuantities().isEmpty())
                        .findFirst()
                        .get()
                        .getMixQuantities()
                        .stream()
                        .findFirst()
                        .get()).getMaterial();

        return base;
    }
}
