package dev.fyloz.trial.colorrecipesexplorer.web.controller.creators;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Company;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.CompanyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;
import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.PagesPaths.CREATOR_COMPANY;
import static dev.fyloz.trial.colorrecipesexplorer.web.PagesPaths.INDEX;

@Controller
public class CompanyCreatorController {

    private CompanyService companyService;

    @Autowired
    public CompanyCreatorController(CompanyService companyService) {
        this.companyService = companyService;
    }

    @GetMapping(CREATOR_COMPANY)
    public ModelAndView showCreationPage(ModelAndView model, Company company) {
        return new ModelResponseBuilder(model)
                .withView(CREATOR_COMPANY)
                .addResponseData(ResponseDataType.COMPANY, company)
                .build();
    }

    /**
     * Permet à l'utilisateur de créer un bannière.
     * <p>
     * La création échouera si:
     * - L'utilisateur n'est pas autorisé à effectuer cette action
     * - La bannière existe déjà
     * - Une erreur est survenue lors de la sauvegarde dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - companyName: Contient le nom de la bannière
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param company La bannière à créer
     * @return La page à afficher.
     */
    @PostMapping(value = CREATOR_COMPANY, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView createCompany(@ModelAttribute @Valid Company company) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder().withRedirect(INDEX);

        if (companyService.isValidForCreation(company)) {
            Optional<Company> savedCompany = companyService.save(company);

            if (savedCompany.isPresent()) {
                return modelResponseBuilder
                        .addResponseData(ResponseDataType.COMPANY_NAME, savedCompany.get().getCompanyName())
                        .build();
            } else {
                modelResponseBuilder.addResponseCode(ResponseCode.ERROR_SAVING);
            }
        } else {
            modelResponseBuilder.addResponseCode(ResponseCode.COMPANY_ALREADY_EXIST, company.getCompanyName());
        }

        return showCreationPage(modelResponseBuilder.build(), company);
    }
}
