const body = document.querySelector("body");
const errorMsgBox = document.querySelector(".errorBox");
const errorMsgBoxText = errorMsgBox.querySelector("p");
const warningMsgBox = document.querySelector(".warningBox");
const warningMsgBoxText = warningMsgBox.querySelector("p");
const successMsgBox = document.querySelector(".successBox");
const successMsgBoxText = successMsgBox.querySelector("p");

$(() => {
    $('.materialCode').each(function () {
        const row = $(this);
        const materialID = row.data("materialid");

        axios.post(`/simdut/${materialID}`)
            .catch(function (err) {
                if (err.response.status === 404) {
                    row.parent().addClass("nosimdut");
                    row.parent().title = simdutNotFoundText;
                }
            });
    });

    $(".messageBox").each(function () {
        checkMessageBoxesDisplay($(this)[0])
    });
});

window.addEventListener("load", () => {

    document.querySelectorAll(".returnButton").forEach((e) => {
        e.addEventListener("click", () => {
            document.location.href = referer;
        });
    });

    document.querySelectorAll(".materialCode").forEach((e) => {
        const materialID = e.getAttribute("data-materialID");
        e.addEventListener("click", () => {
            window.open("/simdut/" + materialID, "_blank");
        });
    });

    document.querySelectorAll("img").forEach((e) => {
        if (e.id !== "logo") {
            e.addEventListener("click", () => {
                window.open(e.src, "_blank");
            });
        }
    });

    document.querySelectorAll(".requireAuth").forEach((e) => {
        e.onsubmit = () => {
            return checkPassword(e);
        };
    });

    document.querySelectorAll(".requireAuth-remover").forEach((e) => {
        e.querySelectorAll(".remover").forEach(elem => {
            elem.addEventListener("click", () => {
                e.action += elem.getAttribute("data-entityID");
                checkPassword(e);
            });
        });
    });

    document.querySelectorAll(".companyTabTitle").forEach(e => {
        e.addEventListener("click", () => {
            const companyName = e.getAttribute("data-companyName");
            const table = document.getElementById("recipes_" + companyName);

            if (table.style.display === "none") {
                table.style.display = "table";
            } else {
                table.style.display = "none";
            }
        });
    });

    document.querySelectorAll('input[type="text"], input[type="number"], input[type="date"], textarea').forEach(e => e.classList.add("rawInput"))

    document.querySelectorAll(".rawInput").forEach(e => e.placeholder = "N/A");

    window.addEventListener("keyup", e => {
        if (e.target) {
            if (e.target.classList.contains("toSave")) {
                warningMsgBoxText.innerText = changesNotSavedText;
                showElement(warningMsgBox);
            }
        }
    });

    // Ajoute le favicon
    let faviconElement = document.createElement("link");
    faviconElement.rel = "icon";
    faviconElement.href = "/favicon.png";
    document.querySelector("head").appendChild(faviconElement);
});

function askDatabaseExport() {
    return confirm(exportAllWarningText);
}

function checkPassword(form, callback) {
    hideElement(errorMsgBox);

    const password = prompt(askPasswordText);
    if (!password) return false;

    let data = {};
    data.password = password;

    axios.post("/password/valid", data)
        .then(r => {
            if (r.data === true) {
                if (form != null) form.submit();
                if (callback != null) callback();
                return true;
            } else {
                errorMsgBoxText.innerText = invalidPasswordText;
                showElement(errorMsgBox);
            }
        })
        .catch(e => {
            errorMsgBoxText.innerText = generalErrorText;
            showElement(errorMsgBox);
            console.log(e);
        });

    return false;
}

function checkMessageBoxesDisplay(element) {
    if (!element.querySelector("p").textContent) hideElement(element);
    else showElement(element);
}

function hideElement(element) {
    element.style.display = "none";
}

function showElement(element) {
    element.style.display = "";
}

const lTomL = 1000;
const galTomL = 3785.41;

let currentUnit = "mL";

// Change les unités selon la sélection de l'utilisateur
function changeUnits(unitSelect, quantitiesSelector, unitsDisplay) {
    currentUnit = unitSelect.value;

    document.querySelectorAll(unitsDisplay).forEach(e => {
        e.innerText = currentUnit;

        // Modifie la quantitée
        const quantityElem = e.parentElement.parentElement.querySelector(quantitiesSelector);
        if (quantityElem) {
            const originalQuantity = parseInt(quantityElem.dataset.quantityml);

            quantityElem.innerText = convertMlToUnit(originalQuantity);
        }
    });
}

// Convertit une quantité en millilitres vers une autre unité de volume
function convertMlToUnit(quantity) {
    let newQuantity = currentUnit === "L" ? quantity / lTomL : currentUnit === "gal" ? quantity / galTomL : quantity;
    return round(newQuantity);
}

// Convertit une quantité d'une autre unité vers des millilitres
function convertUnitToMl(quantity) {
    let newQuantity = currentUnit === "L" ? quantity * lTomL : currentUnit === "gal" ? quantity * galTomL : quantity;
    return round(newQuantity);
}

function round(x) {
    return Math.round(x * 1000) / 1000;
}

function percentageOf(percentage, number) {
    return (percentage / 100) * number;
}

function searchIn(searchString, str) {
    return str.toUpperCase().indexOf(searchString.toUpperCase()) > -1;
}
