// Compteurs des produits
let materialNbr = 0;
// Liste des produits
let materials;

let removeText;
let materialSelectorHtml;
let recipeID;

$(() => {
    recipeID = document.querySelector("#recipeID").value;

    axios.get(`/mix/selector/${recipeID}/-1`)
        .then(r => {
            materialSelectorHtml = r.data;
            init();
        })
        .catch(e => {
            console.log(e.status);
            errorMsgBoxText.innerText = generalErrorText;
            showElement(errorMsgBox);
            console.log(e);
        });
});

$(document).on("click", function () {
    hideMaterialList();
});

$(document).on("click", ".materialSelector", function (event) {
    event.stopPropagation();
    showMaterialList($(this)[0]);
});

$(document).on("click", ".removeMaterial", function () {
    const id = $(this).data("removerow");
    if ($(".materialListRow").length > 1) $(`#${id}`).remove();
});

$(document).on("click", ".upRow", function () {
    const row = $(this).parent().parent();
    row.insertBefore(row.prev());
});

$(document).on("click", ".downRow", function () {
    const row = $(this).parent().parent();
    row.insertBefore(row.next());
});

document.querySelector(".mixMaterialListTitle button").addEventListener("click", () => {
    addMaterial(null, null);
});

function addMaterial(materialCode, quantity) {
    const row = addRow(materialNbr);
    const positionColumn = addColumn("positionButtons", row);
    const materialColumn = addColumn("material", row);
    const quantityColumn = addColumn("quantity", row);
    const unitsColumn = addColumn("units", row);
    const removeButtonColumn = addColumn("removeButton", row);

    addPositionButtons(positionColumn);
    addQuantityInput(quantity, quantityColumn);
    addUnits(unitsColumn);
    addRemoveButton(materialNbr, removeButtonColumn);

    materialColumn.innerHTML = materialSelectorHtml;
    const materialSelector = materialColumn.querySelector("input");
    if (materialCode) {
        const material = materialColumn.querySelector(`.materialList p[data-materialcode="${materialCode}"]`);

        if (material) {
            materialSelector.value = material.dataset.materialcode;
            materialSelector.dataset.usepercentages = material.dataset.usepercentages;
        }
    }

    row.appendChild(materialColumn);
    row.appendChild(quantityColumn);
    row.appendChild(unitsColumn);
    row.appendChild(removeButtonColumn);

    document.querySelector(".mixMaterialList").appendChild(row);

    checkUnits(materialSelector, row);
    materialNbr++;
}

function addRow(index) {
    const row = document.createElement("div");
    row.classList.add("materialListRow");
    row.id = `material_${index}`;
    return row;
}

function addColumn(type, parent) {
    const column = document.createElement("div");
    column.classList.add(type);

    parent.appendChild(column);
    return column;
}

function addUnits(parent) {
    const units = document.createElement("p");
    units.classList.add("quantityUnits");
    units.textContent = "mL";

    parent.appendChild(units);
    return units;
}

function addRemoveButton(index, parent) {
    const button = document.createElement("button");
    button.type = "button";
    button.textContent = removeText;
    button.dataset.removerow = `material_${index}`;
    button.classList.add("removeMaterial");

    parent.appendChild(button);
    return button;
}

function addQuantityInput(quantity, parent) {
    let input = document.createElement("input");

    if (quantity === undefined || quantity === null) quantity = 1;

    input.type = "number";
    input.name = "quantities";
    input.value = quantity;
    input.step = 0.001;
    input.min = 0.001;
    input.required = true;

    parent.appendChild(input);
    return input;
}

function addPositionButtons(parent) {
    const up = document.createElement("button");
    up.classList.add("upRow");
    up.type = "button";
    up.textContent = "↑";
    parent.appendChild(up);

    const down = document.createElement("button");
    down.classList.add("downRow");
    down.type = "button";
    down.textContent = "↓";
    parent.appendChild(down);
}

function showMaterialList(input) {
    hideMaterialList();
    input.parentElement.querySelector(".materialSelector .materialList").classList.add("show");
}

function hideMaterialList() {
    const list = document.querySelector(".materialSelector .materialList.show");

    if (list != null) list.classList.remove("show");
}

function selectMaterial(material) {
    const materialName = material.dataset.materialcode;
    const input = material.parentElement.parentElement.querySelector("input");

    input.value = materialName;

    hideMaterialList();
    checkUnits(material, input.parentElement.parentElement.parentElement)
}

function searchMaterial(input) {
    let filter, filterUpper, materials;

    filter = input.value;
    materials = input.parentElement.querySelectorAll(".materialList p");

    materials.forEach(e => {
        if (searchIn(filter, e.textContent) || searchIn(filter, e.dataset.materialtype)) e.style.display = "";
        else e.style.display = "none";
    });

    const found = input.parentElement.querySelector(`.materialList p[data-materialcode="${filter}"]`);
    if (found) input.dataset.usepercentages = found.dataset.usepercentages;

    checkUnits(input, input.parentElement.parentElement.parentElement);
}

function checkUnits(materialSelector, row) {
    const quantityUnits = row.querySelector(".quantityUnits");
    if (materialSelector.dataset.usepercentages === "true") quantityUnits.innerText = "%";
    else quantityUnits.innerText = "mL";
}
