package dev.fyloz.trial.colorrecipesexplorer.core.services;

import dev.fyloz.trial.colorrecipesexplorer.core.model.MixType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MixTypeService;
import dev.fyloz.trial.colorrecipesexplorer.dao.MixTypeDao;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;

import java.util.Optional;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;

public class MixTypeServiceTests {

    @Mock
    private MaterialTypeService materialTypeService;

    @Mock
    private MixTypeDao mixTypeDao;

    @Spy
    @InjectMocks
    private MixTypeService mixTypeService;

    private MixType testMixType;
    private MaterialType testMaterialType;
    private Material testMaterial;

    @Before
    public void init() {
        MockitoAnnotations.initMocks(this);

        testMaterial = new Material();
        testMaterial.setMaterialID(0);
        testMaterial.setMaterialCode("Test Material");

        testMixType = new MixType();
        testMixType.setTypeID(0);
        testMixType.setTypeName("Test Type");
        testMixType.setMaterial(testMaterial);

        testMaterialType = new MaterialType();
        testMaterialType.setMaterialTypeID(0);
        testMaterialType.setMaterialTypeName("Test Type");
    }

    @Test
    public void getByNameTest() {
        doReturn(Optional.of(testMixType)).when(mixTypeDao).findByTypeName(testMixType.getTypeName());

        Optional<MixType> actual = mixTypeService.getByName(testMixType.getTypeName());
        assertTrue(actual.isPresent());
    }

    @Test
    public void getByNameTestShouldBeEmpty() {
        doReturn(Optional.empty()).when(mixTypeDao).findByTypeName(testMixType.getTypeName());

        Optional<MixType> actual = mixTypeService.getByName(testMixType.getTypeName());
        assertFalse(actual.isPresent());
    }

    @Test
    public void getByNameTestNull() {
        doReturn(Optional.empty()).when(mixTypeDao).findByTypeName(any());

        Optional<MixType> actual = mixTypeService.getByName(null);
        assertFalse(actual.isPresent());
    }

    @Test
    public void getByMaterialTest() {
        doReturn(Optional.of(testMixType)).when(mixTypeDao).findByMaterial(testMaterial);

        Optional<MixType> actual = mixTypeService.getByMaterial(testMaterial);
        assertTrue(actual.isPresent());
    }

    @Test
    public void getByMaterialTestShouldBeEmpty() {
        doReturn(Optional.empty()).when(mixTypeDao).findByMaterial(testMaterial);

        Optional<MixType> actual = mixTypeService.getByMaterial(testMaterial);
        assertFalse(actual.isPresent());
    }

    @Test
    public void getByMaterialTestNull() {
        doReturn(Optional.empty()).when(mixTypeDao).findByMaterial(any());

        Optional<MixType> actual = mixTypeService.getByMaterial(null);
        assertFalse(actual.isPresent());
    }

    @Test
    public void createByNameTest() {
        doReturn(Optional.of(testMaterialType)).when(materialTypeService).getDefaultMaterialType();
        doReturn(Optional.of(testMixType)).when(mixTypeService).save(any());
        doReturn(Optional.empty()).when(mixTypeService).getByName(testMixType.getTypeName());

        Optional<MixType> actual = mixTypeService.createByName(testMixType.getTypeName());
        assertTrue(actual.isPresent());
    }

    @Test
    public void createByNameTestExistsByName() {
        doReturn(Optional.of(testMixType)).when(materialTypeService).getDefaultMaterialType();
        doReturn(Optional.of(testMixType)).when(mixTypeService).save(any());
        doReturn(Optional.of(testMixType)).when(mixTypeService).getByName(testMixType.getTypeName());

        Optional<MixType> actual = mixTypeService.createByName(testMixType.getTypeName());
        verify(mixTypeService, times(0)).save(any());
        assertTrue(actual.isPresent());
    }

    @Test
    public void createByNameTestShouldBeEmpty() {
        doReturn(Optional.empty()).when(materialTypeService).getDefaultMaterialType();
        doReturn(Optional.empty()).when(mixTypeService).save(any());
        doReturn(Optional.empty()).when(mixTypeService).getByName(testMixType.getTypeName());

        Optional<MixType> actual = mixTypeService.createByName(testMixType.getTypeName());
        assertFalse(actual.isPresent());
    }

    @Test
    public void createByNameTestNull() {
        doReturn(Optional.empty()).when(materialTypeService).getDefaultMaterialType();
        doReturn(Optional.empty()).when(mixTypeService).save(any());
        doReturn(Optional.empty()).when(mixTypeService).getByName(any());

        Optional<MixType> actual = mixTypeService.createByName(null);
        assertFalse(actual.isPresent());
    }

}
